@@ -22,13 +22,23 @@ class ASPNET_Viewstate(BadsecretsBase):
 
     def carve_regex(self):
         return re.compile(
-            r"<input.+__VIEWSTATE\"\svalue=\"(.+)\"[\S\s]+<input.+__VIEWSTATEGENERATOR\"\svalue=\"(\w+)\""
+            r"<input.+__VIEWSTATE\"\svalue=\"(.+?)\"[\S\s]+<input.+?__VIEWSTATEGENERATOR\"\svalue=\"(\w+)\""
         )
 
-    def carve_to_check_secret(self, s, url=None):
+    def carve_to_check_secret(self, s, url=None, **kwargs):
         if len(s.groups()) == 2:
-            r = self.check_secret(s.groups()[0], s.groups()[1], url)
-            return r
+            viewstate = s.groups()[0]
+            generator = s.groups()[1]
+            possible_userkey_cookies = ["ASP.NET_SessionId", "__AntiXsrfToken", "ASPSESSIONID"]
+
+            if kwargs.get("cookies") and hasattr(kwargs.get("cookies"), "get"):
+                for cookie_name in possible_userkey_cookies:
+                    if cookie_name in kwargs.get("cookies"):
+                        cookie_value = kwargs.get("cookies").get(cookie_name)
+                        r = self.check_secret(viewstate, generator, url, cookie_value)
+                        if r:
+                            return r
+            return self.check_secret(viewstate, generator, url)
 
     @staticmethod
     def valid_preamble(sourcebytes):
@@ -92,7 +102,8 @@ class ASPNET_Viewstate(BadsecretsBase):
                     else:
                         continue
 
-    def viewstate_validate(self, vkey_bytes, encrypted, viewstate_B64, generator, url, mode):
+    def viewstate_validate(self, vkey_bytes, encrypted, viewstate_B64, generator, url, mode, viewstate_userkey=None):
+
         original_vkey_bytes = vkey_bytes
         viewstate_bytes = base64.b64decode(viewstate_B64)
 
@@ -100,26 +111,35 @@ class ASPNET_Viewstate(BadsecretsBase):
             candidate_hash_algs = list(self.hash_sizes.keys())
         else:
             vs = ViewState(viewstate_B64)
-            try:
-                vs.decode()
-            except ViewStateException:
-                return None
+            vs.decode()
             signature_len = len(vs.signature)
             candidate_hash_algs = self.search_dict(self.hash_sizes, signature_len)
 
+        modifier_bytes = b"\x00" * 4
+        if viewstate_userkey and viewstate_userkey.strip():
+            modifier_bytes += viewstate_userkey.encode("utf-16le")
         for hash_alg in candidate_hash_algs:
             vkey_bytes = original_vkey_bytes
             viewstate_data = viewstate_bytes[: -self.hash_sizes[hash_alg]]
             signature = viewstate_bytes[-self.hash_sizes[hash_alg] :]
             if hash_alg == "MD5":
-                md5_bytes = viewstate_data + vkey_bytes
                 if not encrypted:
-                    md5_bytes += b"\x00" * 4
+                    # if viewstate_userkey and viewstate_userkey.strip():
+                    #    md5_bytes = b"will not work, sorry"
+                    # MD5 + ViewStateUserKey is a horrible edge case that may NEVER work. We will not match on it, currently.
+                    # Last attempt:
+                    # md5_bytes = viewstate_data + vkey_bytes + page_hash_bytes + viewstate_userkey.encode('utf-16le')
+                    # But page_hash_bytes is apparently NOT the generator and my have to be brute-forced.
+                    # Probably not worth it for the 3 servers in the entire world probably using these settings in the wild.
+                    md5_bytes = viewstate_data + vkey_bytes + modifier_bytes
+                else:
+                    md5_bytes = viewstate_data + vkey_bytes
                 h = hashlib.md5(md5_bytes)
             else:
                 vs_data_bytes = viewstate_data
                 if not encrypted:
                     vs_data_bytes += generator
+                    vs_data_bytes += modifier_bytes[4:]
                 if mode == "DOTNET45" and url:
                     s = Simulate_dotnet45_kdf_context_parameters(url)
                     label, context = sp800_108_get_key_derivation_parameters(
@@ -132,7 +152,7 @@ class ASPNET_Viewstate(BadsecretsBase):
                     self.hash_algs[hash_alg],
                 )
 
-            if h.digest() == signature:
+            if signature == h.digest():
                 return hash_alg
 
         return None
@@ -142,6 +162,7 @@ class ASPNET_Viewstate(BadsecretsBase):
         generator_pattern = re.compile(r"^[A-F0-9]{8}$")
 
         url = None
+        viewstate_userkey = None
         generator = "0000"
 
         for arg in args:
@@ -150,14 +171,15 @@ class ASPNET_Viewstate(BadsecretsBase):
                     generator = arg
                 elif url_pattern.match(arg):
                     url = arg
-
+                else:
+                    viewstate_userkey = arg
         # Remove query string from the URL, if any
         if url:
             url = urlsplit(url)._replace(query="").geturl()
-        return generator, url
+        return generator, url, viewstate_userkey
 
     def check_secret(self, viewstate_B64, *args):
-        generator, url = self.resolve_args(args)
+        generator, url, viewstate_userkey = self.resolve_args(args)
 
         if not self.identify(viewstate_B64):
             return None
@@ -166,6 +188,11 @@ class ASPNET_Viewstate(BadsecretsBase):
 
         if self.valid_preamble(base64.b64decode(viewstate_B64)):
             encrypted = False
+            try:
+                vs = ViewState(viewstate_B64)
+                vs.decode()
+            except ViewStateException:
+                return None
         else:
             encrypted = True
 
@@ -180,7 +207,7 @@ class ASPNET_Viewstate(BadsecretsBase):
 
                 for mode in ["DOTNET40", "DOTNET45"]:
                     validationAlgo = self.viewstate_validate(
-                        binascii.unhexlify(vkey), encrypted, viewstate_B64, generator, url, mode
+                        binascii.unhexlify(vkey), encrypted, viewstate_B64, generator, url, mode, viewstate_userkey
                     )
                     if validationAlgo:
                         if encrypted:
@@ -199,6 +226,8 @@ class ASPNET_Viewstate(BadsecretsBase):
                         product_string = f"Viewstate: {viewstate_B64}"
                         if generator != "0000":
                             product_string += f" Generator: {generator[::-1].hex().upper()}"
+                        if viewstate_userkey:
+                            product_string += f" ViewStateUserKey: {viewstate_userkey}"
                         return {"secret": result, "product": product_string, "details": f"Mode [{mode}]"}
         return None
 
